//ADATPED sans analyse()
/*
    This file is part of Rekall.
    Copyright (C) 2013-2015

    Project Manager: Clarisse Bardiot
    Development & interactive design: Guillaume Jacquemin & Guillaume Marais (http://www.buzzinglight.com)

    This file was written by Guillaume Jacquemin.

    Rekall is a free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function Project(url, rekall) {
    this.sources = {};
    this.url = url;
    this.firstAnalysis = true;
    this.metadata = {
        "Title": "",
        "Author": "",
        "Email": "",
        "Date": "",
        "Comments": "",
    };
    var urlItems = rekall.baseUrl.split("/");
    urlItems = $.grep(urlItems, function (n) {
        return (n)
    });
    this.metadata["Title"] = urlItems[urlItems.length - 1];
    this.rekall = rekall;
    this.labels = [];
}

Project.prototype.addDocument = function (key, document) {
    if (this.sources[key] == undefined)
        this.sources[key] = new Source();
    this.sources[key].addDocument(document);
}

Project.prototype.getDocument = function (path) {
    var retour = undefined;
    for (var key in this.sources) {
        if (retour == undefined)
            retour = this.sources[key].getDocument(path);
    }
    return retour;
}

Project.prototype.loadXML = function (xml) {
    this.sources["Files"] = new Source();

    let thiss = this;
    let counts = {documents: 0, tags: 0, metadatas: 0};
    xml.find('document').each(function () {
        if (($(this).attr("remove") != undefined) && ($(this).attr("key") != undefined) && ($(this).attr("remove") == "true")) {
            var rekallDoc = thiss.sources["Files"].documents[$(this).attr("key")];
            if (rekallDoc != undefined) {
                for (var tagIndex in rekallDoc.tags)
                    rekallDoc.tags[tagIndex].visuel.rect.remove();
                delete this.rekall.project.sources["Files"].documents[rekallDoc.key];
            }
        } else {
            let rekallDoc = new Document({project: thiss});
            counts.documents++;
            counts.tags++;
            $(this).find('meta').each(function () {
                let rekallDocMeta = new Metadata();
                rekallDocMeta.content = $(this).attr('cnt');
                rekallDocMeta.metadataKey = $(this).attr('ctg');
                rekallDoc.setMetadata(rekallDocMeta);
                counts.metadatas++;
            });
            if ($(this).attr("key") != undefined)
                rekallDoc.key = $(this).attr("key");
            thiss.addDocument("Files", rekallDoc);
        }
    });
    xml.find('edition').each(function () {
        let key = $(this).attr('key');
        if (thiss.sources["Files"].documents[key] != undefined) {
            var version = $(this).attr('version');
            var metadataKey = $(this).attr('metadataKey');
            var metadataValue = $(this).attr('metadataValue');
            thiss.sources["Files"].documents[key].setMetadata(metadataKey, metadataValue, version);
        }
    });
    xml.find('tag').each(function () {
        let key = $(this).attr('key');
        if (thiss.sources["Files"].documents[key] != undefined) {
            let version = $(this).attr('version');
            let timeStart = parseFloat($(this).attr('timeStart')) + 0.;
            let timeEnd = parseFloat($(this).attr('timeEnd')) + 0.;
            for (var index in thiss.sources["Files"].documents[key].tags) {
                thiss.sources["Files"].documents[key].tags[index].setTimeStart(timeStart);
                thiss.sources["Files"].documents[key].tags[index].setTimeEnd(timeEnd);
            }
        }
    });
    xml.find('projectMeta').each(function () {
        thiss.metadata[$(this).attr('ctg')] = $(this).attr('cnt');
    });
    let videoUrl = undefined, videoTech = undefined;
    xml.find('video').each(function () {
        videoUrl = $(this).attr('url');
        videoTech = $(this).attr('tech');
    });

    if (this.rekall.ensureVideoPlayerCreated(videoUrl, videoTech)) {
        $(window).trigger("resize");
    }

    this.analyse();
}

Project.prototype.timelineUpdate = function () {

}

Project.prototype.analyse = function () {
    $('#flattentimeline').html("<div id='flattentimeline_highlight'></div>");

    // let parsed_labels = $('input.annotation_labels').toArray().map(i => i.value);
    let parsed_labels = this.labels;

    //Analyse
    Tags.reset();
    this.rekall.sortings["horizontal"].analyseStart();
    this.rekall.sortings["colors"].analyseStart();
    for (let keySource in this.sources) {
        for (let keyDocument in this.sources[keySource].documents) {
            for (let key in this.sources[keySource].documents[keyDocument].tags) {
                let tag = this.sources[keySource].documents[keyDocument].tags[key];
                this.rekall.sortings["horizontal"].analyseAdd(tag);
                this.rekall.sortings["colors"].analyseAdd(tag);
                Tags.push(tag);
                tag.getLabels().forEach(l => {
                    if (parsed_labels.indexOf(l) === -1) {
                        parsed_labels.push(l);
                    }
                })
            }
        }
    }
    this.rekall.sortings["horizontal"].analyseEnd();
    this.rekall.sortings["colors"].analyseEnd();
    Tags.sort();

    //Affichage
    let categories = rekall.sortings["horizontal"].categories;
    if (this.rekall.sortings["horizontal"].metadataKey == "Time")
        categories = {time: {tags: Tags.getFlattenTimeLineTags()}};

    //Affectation des couleurs
    for (let key in this.rekall.sortings["colors"].categories) {
        let colorSortingCategory = this.rekall.sortings["colors"].categories[key];
        for (let tagKey in colorSortingCategory.tags) {
            let tag = colorSortingCategory.tags[tagKey];
            tag.update(colorSortingCategory.color);
            tag.isSelectable = colorSortingCategory.checked;
        }
    }


    //Tags / catégories
    let markers = [], captions = [];
    for (let key in categories) {
        $.each(categories[key].tags, function (index, tag) {
            //Elements sur la timeline
            markers.push({
                time: tag.getTimeStart() + 0.1,
                text: tag.getMetadata("Rekall->Name"),
                overlayText: tag.getMetadata("Rekall->Comments"),
                css: {
                    "background-color": tag.color,
                    "width": "2px",
                },
                markerTipCss: {
                    "font-familly": "OpenSans",
                    "color": tag.color,
                },
                onMarkerClick: function () {
                    tag.openPopupEdit();
                }
            });

            //Captions en popup
            if (tag.isMarker()) {
                captions.push({
                    startTime: tag.getTimeStart() * 1000,
                    endTime: (tag.timeStart + max(2, tag.timeEnd - tag.timeStart)) * 1000,
                    position: "HB",
                    data: tag.getMetadata("Rekall->Name"),
                    alignment: "C",
                    css: {
                        "background-color": "black",
                        "font-familly": "OpenSans",
                        "color": tag.color,
                    },
                    onCaptionChange: function () {
                        tag.openPopupEdit();
                    }
                });
            }

            //Analyse de vignettes
            let thumbUrl = Utils.getPreviewPath(tag);
            tag.thumbnail = {url: thumbUrl, tag: tag};

            $('#flattentimeline').append(tag.createTimelineDiv.call(tag));

            //Ouverture du popup
            if ((Tag.keyToOpenAfterLoading != undefined) && (tag.document.key == Tag.keyToOpenAfterLoading)) {
                tag.openPopupEdit();
                Tag.keyToOpenAfterLoading = undefined;
            }
        });
    }
    this.rekall.timeline.updateFlattenTimeline();

    //Initialisation des captions
    if (this.rekall.videoPlayer.caption.updateCaption == undefined) {
        this.rekall.videoPlayer.caption({
            data: captions,
            setting: {
                captionSize: 3,
                captionStyle: {
                    'background-color': 'rgba(255,0,0,0.8)',
                    'color': 'white',
                    'padding': '3px',
                    'font-family': 'OpenSans',
                },
                onCaptionChange: function (num_c) {
                }
            }
        });
    } else if ((captions) && (captions.length > 0)) {
        this.rekall.videoPlayer.caption.loadNewCaption({data: captions});
    }

    //Ajout des marqueurs sur la timeline
    if (this.rekall.videoPlayer.markers.removeAll == undefined) {
        this.rekall.videoPlayer.markers({
            markerStyle: {
                'width': '7px',
                'border-radius': '30%',
                'background-color': 'yellow'
            },
            markerTip: {
                display: true,
                text: function (marker) {
                    return marker.text;
                },
                time: function (marker) {
                    return marker.time;
                }
            },
            breakOverlay: {
                display: false,
                displayTime: 3,
                style: {
                    'width': '100%',
                    'height': '20%',
                    'background-color': 'rgba(0,0,0,0.7)',
                    'color': 'white',
                    'font-size': '17px'
                },
                text: function (marker) {
                    return "Break overlay: " + marker.overlayText;
                }
            },
            onMarkerClick: function (marker) {
            },
            onMarkerReached: function (marker) {
            },
            markers: markers
        });
    } else {
        this.rekall.videoPlayer.markers.reset(markers);
    }
}
