<?php

namespace App\Controller;

use App\Helper\LegacyHelper;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\File\Exception\FileNotFoundException;

class FallbackController extends AbstractController
{
    #[Route('/', name:'get_legacy_resource', priority:-2)]
    #[Route('/{controller}', name:'get_legacy_resource', requirements:['controller' => '.+'], priority:-1)]
    public function getLegacyResourceAction(Request $request, ?string $controller = null): Response
    {
        if ($request->query->has('p') || $request->request->has('p')) {
            $this->denyAccessUnlessGranted('IS_AUTHENTICATED_REMEMBERED');
        }

        if (null === $controller) {
            return LegacyHelper::transferToLegacy(
                $request,
                strval($this->getParameter('app.legacy_external_prefix')),
                strval($this->getParameter('app.legacy_url'))
            );
        }

        // use iframe to enhance speed but not for creation
        if (! str_starts_with($controller, 'legacy')) {
            $original_controller = $request->getPathInfo();
            $original_query_string = $request->getQueryString();

            $extension = strrchr($original_controller, '.');
            if ($extension != '.html' && $extension != '.php' && $extension != false) {
                //$url = "{$path_to_legacy_code}{$originalController}";
                $url = "{$original_controller}";
                try {
                    $mime = LegacyHelper::getMime($url, $extension);
                    $response = new BinaryFileResponse($url);
                    $response->headers->set('Content-Type', $mime ?: 'application/octet-stream');
                    return $response;
                } catch (FileNotFoundException $e) {
                    // try normal access by url
                }
            }
            $separator = '';
            if (!$extension) {
                $separator = '/';
            }

            $app_legacy_url_external = strval($this->getParameter('app.legacy_url_external'));

            $url = "{$app_legacy_url_external}/" .
                "{$original_controller}{$separator}?{$original_query_string}";
            $url = preg_replace('(^https?:\/\/[^/]+(:\d+)?)', '', $url);

            $pattern = '/\/+/i';
            /** @phpstan-ignore-next-line */
            $url = preg_replace($pattern, '/', $url);

            return $this->render("project/edit.html.twig", [
                'url' => $url
            ]);
        }

        return LegacyHelper::transferToLegacy(
            $request,
            strval($this->getParameter('app.legacy_external_prefix')),
            strval($this->getParameter('app.legacy_url'))
        );
    }
}
